// Copyright 2017 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

package com.qpc.qpc_admob;

import android.app.Activity;
import android.util.Log;
import android.util.SparseArray;
import android.view.Gravity;
import android.view.View;
import android.view.ViewGroup;
import android.view.WindowManager;
import android.widget.LinearLayout;
import android.widget.Toast;

import com.bytedance.sdk.openadsdk.AdSlot;
import com.bytedance.sdk.openadsdk.TTAdDislike;
import com.bytedance.sdk.openadsdk.TTAdManager;
import com.bytedance.sdk.openadsdk.TTAdNative;
import com.bytedance.sdk.openadsdk.TTAdSdk;
import com.bytedance.sdk.openadsdk.TTAppDownloadListener;
import com.bytedance.sdk.openadsdk.TTBannerAd;
import com.bytedance.sdk.openadsdk.TTInteractionAd;
import com.bytedance.sdk.openadsdk.TTNativeExpressAd;
import com.bytedance.sdk.openadsdk.TTNativeExpressAd.ExpressAdInteractionListener;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import io.flutter.plugin.common.MethodChannel;

abstract class QsjMobileAd extends MobileAd implements ExpressAdInteractionListener, TTAdNative.InteractionAdListener, TTNativeExpressAd.AdInteractionListener, TTAdNative.NativeExpressAdListener,TTAppDownloadListener {
    private static final String TAG = "flutter";

    TTAdNative mTTAdNative;
    TTInteractionAd ttInteractionAd;
    LinearLayout content ;



    private QsjMobileAd(int id, Activity activity, MethodChannel channel) {
     super(id,activity,channel);
    }

    static QsjBanner createBanner(Integer id, String adUnitId, int width, int height, Activity activity, MethodChannel channel) {
        MobileAd ad = getAdForId(id);
        return (ad != null) ? (QsjBanner) ad : new QsjBanner(id,width,height,activity,channel);

    }

    static QsjInterstitial createInterstitial(Integer id, Activity activity, MethodChannel channel) {
        MobileAd ad = getAdForId(id);
        return (ad != null) ? (QsjInterstitial) ad : new QsjInterstitial(id, activity, channel);
    }


    @Override
    public void onAdClicked(View view, int type) {
//    TToast.show(mContext, "广告被点击");
        channel.invokeMethod("onAdClicked", argumentsMap());
    }

    @Override
    public void onAdShow(View view, int type) {
//    TToast.show(mContext, "广告展示");
        boolean statusWasPending = status == Status.PENDING;
        status = Status.LOADED;
        channel.invokeMethod("onAdLoaded", argumentsMap());
        if (statusWasPending) show();

    }

    @Override
    public void onAdDismiss() {
        Log.d(TAG, "插屏广告消失");
        TToast.show(activity, "广告消失");
    }

    @Override
    public void onRenderFail(View view, String msg, int code) {
        Log.w(TAG, msg + code);
        status = Status.FAILED;
        channel.invokeMethod("onAdFailedToLoad", argumentsMap("errorMsg",msg,"errorCode", code));
    }



    @Override
    public void onIdle() {
        TToast.show(activity, "点击开始下载", Toast.LENGTH_LONG);
    }

    @Override
    public void onDownloadActive(long totalBytes, long currBytes, String fileName, String appName) {

        TToast.show(activity, "下载中，点击暂停", Toast.LENGTH_LONG);

    }

    @Override
    public void onDownloadPaused(long totalBytes, long currBytes, String fileName, String appName) {
        TToast.show(activity, "下载暂停，点击继续", Toast.LENGTH_LONG);
    }

    @Override
    public void onDownloadFailed(long totalBytes, long currBytes, String fileName, String appName) {
        TToast.show(activity, "下载失败，点击重新下载", Toast.LENGTH_LONG);
    }

    @Override
    public void onInstalled(String fileName, String appName) {
        TToast.show(activity, "安装完成，点击图片打开", Toast.LENGTH_LONG);
    }

    @Override
    public void onDownloadFinished(long totalBytes, String fileName, String appName) {
        TToast.show(activity, "点击安装", Toast.LENGTH_LONG);
    }


    @Override
    public void onError(int code, String message) {
        TToast.show(activity, "load error : " + code + ", " + message);
    }

    @Override
    public void onInteractionAdLoad(TTInteractionAd ttInteractionAd) {
        TToast.show(activity, "type:  " + ttInteractionAd.getInteractionType());
        ttInteractionAd.setAdInteractionListener((TTInteractionAd.AdInteractionListener) this);
        ttInteractionAd.setDownloadListener(this);
        this.ttInteractionAd = ttInteractionAd;
        ttInteractionAd.showInteractionAd(activity);

    }

    @Override
    public void onNativeExpressAdLoad(List<TTNativeExpressAd> ads) {
        TToast.show(activity, "load ads : " + ads.size() );
        if (ads == null || ads.size() == 0){
            TToast.show(activity, "load ads : " + 0 );
            return;
        }
        TToast.show(activity, "load ads : " + ads.size() );
        TTNativeExpressAd mTTAd = ads.get(0);
        View view = mTTAd.getExpressAdView();
        content = new LinearLayout(activity);
        content.setId(id);
        content.setOrientation(LinearLayout.VERTICAL);
//        content.setGravity(anchorType);
        content.setMinimumHeight(100);
        content.setMinimumWidth(300);

        view.bringToFront();
        view.setActivated(true);
        view.setVisibility(View.VISIBLE);
        content.addView(view);

        final float scale = activity.getResources().getDisplayMetrics().density;
//
//        int left = horizontalCenterOffset > 0 ? (int) (horizontalCenterOffset * scale) : 0;
//        int right =
//                horizontalCenterOffset < 0 ? (int) (Math.abs(horizontalCenterOffset) * scale) : 0;
//        if (anchorType == Gravity.BOTTOM) {
//            content.setPadding(left, 0, right, (int) (anchorOffset * scale));
//        } else {
//            content.setPadding(left, (int) (anchorOffset * scale), right, 0);
//        }

        activity.addContentView(
                content,
                new ViewGroup.LayoutParams(
                        ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT));

        mTTAd.setSlideIntervalTime(30*1000);//设置轮播间隔 ms,不调用则不进行轮播展示
        mTTAd.setExpressInteractionListener(this);
        mTTAd.setDownloadListener(this);
        mTTAd.render();//调用render开始渲染广告


    }

    @Override
    public void onRenderSuccess(View view, float width, float height) {
        //返回view的宽高 单位 dp
        TToast.show(activity, "渲染成功haha");
        //在渲染成功回调时展示广告，提升体验
//    mExpressContainer.removeAllViews();
//    mExpressContainer.addView(view)

        view.bringToFront();



    }

    static class QsjBanner extends QsjMobileAd {

        private  int width;
        private int height;


        private QsjBanner(Integer id, int width, int height, Activity activity, MethodChannel channel) {

            super(id, activity, channel);
            activity.getWindow().addFlags(WindowManager.LayoutParams.FLAG_SHOW_WHEN_LOCKED
                    | WindowManager.LayoutParams.FLAG_TURN_SCREEN_ON);
            this.width=width;
            this.height=height;
            //step5:请求广告
            TTAdManager ttAdManager =  TTAdSdk.getAdManager();
            //step2:(可选，强烈建议在合适的时机调用):申请部分权限，如read_phone_state,防止获取不了imei时候，下载类广告没有填充的问题。
            ttAdManager.requestPermissionIfNecessary(activity);
            //step3:创建TTAdNative对象,用于调用广告请求接口
            mTTAdNative = ttAdManager.createAdNative(activity);


        }

        @Override
        void load(String adUnitId, Map<String, Object> targetingInfo) {

            //step4:创建广告请求参数AdSlot,具体参数含义参考文档
            AdSlot adSlot = new AdSlot.Builder()
                    .setCodeId(adUnitId) //广告位id
                    .setSupportDeepLink(true)
                    .setImageAcceptedSize(600, 257)
                    .build();

            mTTAdNative.loadBannerExpressAd(adSlot, new TTAdNative.NativeExpressAdListener() {
                @Override
                public void onError(int code, String message) {
                    TToast.show(activity, "load error : " + code + ", " + message);
//                    mExpressContainer.removeAllViews();
                }

                @Override
                public void onNativeExpressAdLoad(List<TTNativeExpressAd> ads) {

                    TToast.show(activity, "load ads : " + ads.size() );

                    if (ads == null || ads.size() == 0){
                        return;
                    }
                    TTNativeExpressAd mTTAd = ads.get(0);
//                mTTAd.setSlideIntervalTime(30*1000);
//                    bindAdListener(mTTAd);
//                    startTime = System.currentTimeMillis();
                    mTTAd.render();
                    activity.addContentView(
                            mTTAd.getExpressAdView(),
                            new ViewGroup.LayoutParams(
                                    ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT));
                    activity.recreate();

                }
            });
            //step5:请求广告，对请求回调的广告作渲染处理
//            mTTAdNative.loadBannerAd(adSlot, new TTAdNative.BannerAdListener() {
//
//                @Override
//                public void onError(int code, String message) {
//                    TToast.show(activity, "load error : " + code + ", " + message);
////                    mBannerContainer.removeAllViews();
//                }
//
//                @Override
//                public void onBannerAdLoad(final TTBannerAd ad) {
//
//                    TToast.show(activity, "TTBannerAd load");
//                    if (ad == null) {
//                        return;
//                    }
//                    View bannerView = ad.getBannerView();
//                    if (bannerView == null) {
//                        return;
//                    }
//                    //设置轮播的时间间隔  间隔在30s到120秒之间的值，不设置默认不轮播
//                    ad.setSlideIntervalTime(30 * 1000);
////                    mBannerContainer.removeAllViews();
////                    mBannerContainer.addView(bannerView);
//                    content = new LinearLayout(activity);
//                    content.addView(bannerView);
//
//                    //设置广告互动监听回调
//                    ad.setBannerInteractionListener(new TTBannerAd.AdInteractionListener() {
//                        @Override
//                        public void onAdClicked(View view, int type) {
//                            TToast.show(activity, "广告被点击");
//                        }
//
//                        @Override
//                        public void onAdShow(View view, int type) {
//                            TToast.show(activity, "广告展示");
//                        }
//                    });
//                    //（可选）设置下载类广告的下载监听
//
//                    //在banner中显示网盟提供的dislike icon，有助于广告投放精准度提升
//                    ad.setShowDislikeIcon(new TTAdDislike.DislikeInteractionCallback() {
//                        @Override
//                        public void onSelected(int position, String value) {
//                            TToast.show(activity, "点击 " + value);
//                            //用户选择不喜欢原因后，移除广告展示
////                            mBannerContainer.removeAllViews();
//                        }
//
//                        @Override
//                        public void onCancel() {
//                            TToast.show(activity, "点击取消 ");
//                        }
//                    });
//
//                    //获取网盟dislike dialog，您可以在您应用中本身自定义的dislike icon 按钮中设置 mTTAdDislike.showDislikeDialog();
//                /*mTTAdDislike = ad.getDislikeDialog(new TTAdDislike.DislikeInteractionCallback() {
//                        @Override
//                        public void onSelected(int position, String value) {
//                            TToast.show(mContext, "点击 " + value);
//                        }
//
//                        @Override
//                        public void onCancel() {
//                            TToast.show(mContext, "点击取消 ");
//                        }
//                    });
//                if (mTTAdDislike != null) {
//                    XXX.setOnClickListener(new View.OnClickListener() {
//                        @Override
//                        public void onClick(View v) {
//                            mTTAdDislike.showDislikeDialog();
//                        }
//                    });
//                } */
//
//                }
//            });
        }

        @Override
        void show() {
            if (status == Status.LOADING) {
                status = Status.PENDING;
                return;
            }
            if (status != Status.LOADED) return;

        }

        @Override
        void dispose() {
            super.dispose();


            View contentView = activity.findViewById(id);
            if (contentView == null || !(contentView.getParent() instanceof ViewGroup)) return;

            ViewGroup contentParent = (ViewGroup) (contentView.getParent());
            contentParent.removeView(contentView);
            mTTAdNative=null;
        }
    }

    static class QsjInterstitial extends QsjMobileAd {

        private QsjInterstitial(int id, Activity activity, MethodChannel channel) {
            super(id, activity, channel);
            //step5:请求广告
            TTAdManager ttAdManager =  TTAdSdk.getAdManager();
            //step2:(可选，强烈建议在合适的时机调用):申请部分权限，如read_phone_state,防止获取不了imei时候，下载类广告没有填充的问题。
            ttAdManager.requestPermissionIfNecessary(activity);
            //step3:创建TTAdNative对象,用于调用广告请求接口
            mTTAdNative = ttAdManager.createAdNative(activity);
        }

        @Override
        void load(String adUnitId, Map<String, Object> targetingInfo) {
            status = Status.LOADING;

            AdSlot adSlot = new AdSlot.Builder()
                    .setCodeId(adUnitId)
                    .setSupportDeepLink(true)
                    .setAdCount(1)
                    .build();
            mTTAdNative.loadInteractionExpressAd(adSlot, (TTAdNative.NativeExpressAdListener) this);
        }

        @Override
        void show() {
            if (status == Status.LOADING) {
                status = Status.PENDING;
                return;
            }
            if(ttInteractionAd !=null ){
                ttInteractionAd.showInteractionAd(activity);
            }

        }

        // It is not possible to hide/remove/destroy an AdMob interstitial Ad.
    }
}
